from skimage import measure
from shapely.geometry import Polygon, MultiPolygon
import numpy as np
from pycocotools import coco
from matplotlib import pyplot as plt
import cv2


def create_sub_mask_annotation(sub_mask, category_id, annotation_id, image_id=0, is_crowd=False):
    contours = measure.find_contours(sub_mask, 0.5, positive_orientation='low')
    segmentations = []
    polygons = []
    for contour in contours:
        # Flip from (row, col) representation to (x, y)
        # and subtract the padding pixel
        for i in range(len(contour)):
            row, col = contour[i]
            contour[i] = (col - 1, row - 1)

        # Make a polygon and simplify it
        poly = Polygon(contour)
        poly = poly.simplify(1.0)
        polygons.append(poly)
        try:
            segmentation = np.array(poly.exterior.coords).ravel().tolist()
            segmentations.append(segmentation)
        except Exception as e:
            print(e)
            pass

    # Combine the polygons to calculate the bounding box and area
    multi_poly = MultiPolygon(polygons)

    if multi_poly.area < 150 or len(segmentations) == 0:
        return []

    x, y, max_x, max_y = multi_poly.bounds
    width = max_x - x
    height = max_y - y
    bbox = (x, y, width, height)
    area = multi_poly.area

    annotation = {
        'segmentation': segmentations,
        'iscrowd': is_crowd,
        'category_id': category_id,
        'id': annotation_id,
        'image_id': image_id,
        'bbox': bbox,
        'area': area
    }

    return annotation


def preview_img_and_target(img, prediction):
    # prediction: [{'boxes': tensor([[]]), 'labels': tensor([]), 'masks': tensor([[[[ ]]]])}]
    coco_obj = coco.COCO()
    coco_obj.dataset = prediction_to_coco_dict(prediction)
    coco_obj.createIndex()

    plt.figure(figsize=(img.size[0]/80, img.size[1]/80))
    plt.imshow(img)
    plt.axis('off')
    annIds = coco_obj.getAnnIds(iscrowd=None)
    anns = coco_obj.loadAnns(annIds)
    coco_obj.showAnns(anns, draw_bbox=True)
    plt.show()
    plt.clf()


def prediction_to_coco_dict(prediction):
    anns = {}
    annotations = []
    for i, pred in enumerate(prediction):
        mask = pred["masks"][0, 0].mul(255).byte().cpu().numpy()
        _, mask = cv2.threshold(mask, 127, 255, cv2.THRESH_BINARY)
        annotations.append(create_sub_mask_annotation(mask, pred["labels"][0], i))

    anns["annotations"] = annotations
    return anns

# test_img_path = os.path.join(data_dir, img_dict["file_name"])
# img = Image.open("data/raw/images/207.png")
#
# preview_img_and_target(img, prediction)
